from typing import Any, Dict, Optional
from .base_model import Upload
from .custom_fields import AbcampaignFields, ApiKeyFields, CompletionFields, CustomRecipeFields, DatasetFields, DatasetUploadProcessingStatusFields, DatasetValidationOutputFields, DeleteConfirmFields, DirectFeedbackFields, GraderFields, JobFields, JudgeFields, MetricFields, MetricWithContextFields, ModelFields, ModelServiceFields, RemoteEnvFields, RoleFields, SystemPromptTemplateFields, TeamFields, TeamMemberFields, ToolProviderFields, UseCaseFields, UserFields
from .custom_typing_fields import GraphQLField, RemoteEnvTestUnion
from .input_types import AbcampaignCreate, AddExternalModelInput, AddHFModelInput, AddModelInput, ApiKeyCreate, AttachModel, CancelAllocationInput, CreateRecipeInput, CreateToolProviderInput, DatasetCreate, DatasetCreateFromFilters, DatasetCreateFromMultipartUpload, FeedbackAddInput, FeedbackUpdateInput, GraderCreateInput, GraderUpdateInput, JobInput, JudgeCreate, JudgeUpdate, MetricCreate, MetricLink, MetricUnlink, ModelComputeConfigInput, ModelPlacementInput, ModelServiceDisconnect, PrebuiltJudgeCreate, RemoteEnvCreate, ResizePartitionInput, RoleCreate, SystemPromptTemplateCreate, SystemPromptTemplateUpdate, TeamCreate, TeamMemberRemove, TeamMemberSet, UpdateCompletion, UpdateModelService, UpdateRecipeInput, UpdateToolProviderInput, UseCaseCreate, UseCaseShares, UseCaseUpdate, UserCreate

class Mutation:
    """@private"""

    @classmethod
    def create_ab_campaign(cls, input: AbcampaignCreate) -> AbcampaignFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'AbcampaignCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return AbcampaignFields(field_name='createAbCampaign', arguments=cleared_arguments)

    @classmethod
    def cancel_ab_campaign(cls, input: str) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'IdOrKey!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='cancelAbCampaign', arguments=cleared_arguments)

    @classmethod
    def resize_inference_partition(cls, input: ResizePartitionInput) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'ResizePartitionInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='resizeInferencePartition', arguments=cleared_arguments)

    @classmethod
    def cancel_allocation(cls, input: CancelAllocationInput) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'CancelAllocationInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='cancelAllocation', arguments=cleared_arguments)

    @classmethod
    def create_custom_recipe(cls, use_case: str, input: CreateRecipeInput, file: Upload) -> CustomRecipeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'input': {'type': 'CreateRecipeInput!', 'value': input}, 'file': {'type': 'Upload!', 'value': file}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return CustomRecipeFields(field_name='createCustomRecipe', arguments=cleared_arguments)

    @classmethod
    def update_custom_recipe(cls, use_case: str, id: str, input: UpdateRecipeInput, *, file: Optional[Upload]=None) -> CustomRecipeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'id': {'type': 'IdOrKey!', 'value': id}, 'input': {'type': 'UpdateRecipeInput!', 'value': input}, 'file': {'type': 'Upload', 'value': file}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return CustomRecipeFields(field_name='updateCustomRecipe', arguments=cleared_arguments)

    @classmethod
    def delete_custom_recipe(cls, use_case: str, id: str) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'id': {'type': 'IdOrKey!', 'value': id}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='deleteCustomRecipe', arguments=cleared_arguments)

    @classmethod
    def create_dataset_from_multipart_upload(cls, input: DatasetCreateFromMultipartUpload) -> DatasetUploadProcessingStatusFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'DatasetCreateFromMultipartUpload!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DatasetUploadProcessingStatusFields(field_name='createDatasetFromMultipartUpload', arguments=cleared_arguments)

    @classmethod
    def create_dataset(cls, input: DatasetCreate, file: Upload) -> DatasetFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'DatasetCreate!', 'value': input}, 'file': {'type': 'Upload!', 'value': file}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DatasetFields(field_name='createDataset', arguments=cleared_arguments)

    @classmethod
    def create_dataset_from_filters(cls, input: DatasetCreateFromFilters) -> DatasetFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'DatasetCreateFromFilters!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DatasetFields(field_name='createDatasetFromFilters', arguments=cleared_arguments)

    @classmethod
    def delete_dataset(cls, id_or_key: str, use_case: str) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='deleteDataset', arguments=cleared_arguments)

    @classmethod
    def update_completion(cls, input: UpdateCompletion) -> CompletionFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'UpdateCompletion!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return CompletionFields(field_name='updateCompletion', arguments=cleared_arguments)

    @classmethod
    def create_system_prompt_template(cls, input: SystemPromptTemplateCreate) -> SystemPromptTemplateFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'SystemPromptTemplateCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return SystemPromptTemplateFields(field_name='createSystemPromptTemplate', arguments=cleared_arguments)

    @classmethod
    def derive_system_prompt_template(cls, input: SystemPromptTemplateUpdate) -> SystemPromptTemplateFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'SystemPromptTemplateUpdate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return SystemPromptTemplateFields(field_name='deriveSystemPromptTemplate', arguments=cleared_arguments)

    @classmethod
    def create_job(cls, input: JobInput) -> JobFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'JobInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return JobFields(field_name='createJob', arguments=cleared_arguments)

    @classmethod
    def cancel_job(cls, id: Any) -> JobFields:
        arguments: Dict[str, Dict[str, Any]] = {'id': {'type': 'UUID!', 'value': id}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return JobFields(field_name='cancelJob', arguments=cleared_arguments)

    @classmethod
    def create_metric(cls, input: MetricCreate) -> MetricFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'MetricCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return MetricFields(field_name='createMetric', arguments=cleared_arguments)

    @classmethod
    def link_metric(cls, input: MetricLink) -> MetricWithContextFields:
        """Link a metric and a use case. Can also be used to update the `isPinned`"""
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'MetricLink!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return MetricWithContextFields(field_name='linkMetric', arguments=cleared_arguments)

    @classmethod
    def unlink_metric(cls, input: MetricUnlink) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'MetricUnlink!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='unlinkMetric', arguments=cleared_arguments)

    @classmethod
    def attach_model(cls, input: AttachModel) -> ModelServiceFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'AttachModel!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ModelServiceFields(field_name='attachModel', arguments=cleared_arguments)

    @classmethod
    def disconnect_model(cls, input: ModelServiceDisconnect) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'ModelServiceDisconnect!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='disconnectModel', arguments=cleared_arguments)

    @classmethod
    def update_model_service(cls, input: UpdateModelService) -> ModelServiceFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'UpdateModelService!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ModelServiceFields(field_name='updateModelService', arguments=cleared_arguments)

    @classmethod
    def deploy_model(cls, id_or_key: str, wait: bool, *, placement: Optional[ModelPlacementInput]=None) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'wait': {'type': 'Boolean!', 'value': wait}, 'placement': {'type': 'ModelPlacementInput', 'value': placement}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='deployModel', arguments=cleared_arguments)

    @classmethod
    def terminate_model(cls, id_or_key: str, force: bool) -> GraphQLField:
        """If a model is used by several use cases with `attached = true`, you need to specify 'force = true' to be able to deactivate the model"""
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'force': {'type': 'Boolean!', 'value': force}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='terminateModel', arguments=cleared_arguments)

    @classmethod
    def add_external_model(cls, input: AddExternalModelInput) -> ModelFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'AddExternalModelInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ModelFields(field_name='addExternalModel', arguments=cleared_arguments)

    @classmethod
    def add_model(cls, input: AddModelInput) -> ModelFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'AddModelInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ModelFields(field_name='addModel', arguments=cleared_arguments)

    @classmethod
    def import_hf_model(cls, input: AddHFModelInput) -> JobFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'AddHFModelInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return JobFields(field_name='importHfModel', arguments=cleared_arguments)

    @classmethod
    def update_model_compute_config(cls, id_or_key: str, input: ModelComputeConfigInput) -> ModelFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'input': {'type': 'ModelComputeConfigInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ModelFields(field_name='updateModelComputeConfig', arguments=cleared_arguments)

    @classmethod
    def add_remote_env(cls, input: RemoteEnvCreate) -> RemoteEnvFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'RemoteEnvCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return RemoteEnvFields(field_name='addRemoteEnv', arguments=cleared_arguments)

    @classmethod
    def remove_remote_env(cls, id_or_key: str) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='removeRemoteEnv', arguments=cleared_arguments)

    @classmethod
    def test_remote_env(cls, input: RemoteEnvCreate) -> RemoteEnvTestUnion:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'RemoteEnvCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return RemoteEnvTestUnion(field_name='testRemoteEnv', arguments=cleared_arguments)

    @classmethod
    def validate_dataset_schema(cls, remote_env: str, dataset: str, usecase: str) -> DatasetValidationOutputFields:
        arguments: Dict[str, Dict[str, Any]] = {'remoteEnv': {'type': 'IdOrKey!', 'value': remote_env}, 'dataset': {'type': 'IdOrKey!', 'value': dataset}, 'usecase': {'type': 'IdOrKey!', 'value': usecase}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DatasetValidationOutputFields(field_name='validateDatasetSchema', arguments=cleared_arguments)

    @classmethod
    def create_use_case(cls, input: UseCaseCreate) -> UseCaseFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'UseCaseCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return UseCaseFields(field_name='createUseCase', arguments=cleared_arguments)

    @classmethod
    def update_use_case(cls, id_or_key: str, input: UseCaseUpdate) -> UseCaseFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'input': {'type': 'UseCaseUpdate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return UseCaseFields(field_name='updateUseCase', arguments=cleared_arguments)

    @classmethod
    def share_use_case(cls, id_or_key: str, input: UseCaseShares) -> UseCaseFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'input': {'type': 'UseCaseShares!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return UseCaseFields(field_name='shareUseCase', arguments=cleared_arguments)

    @classmethod
    def create_api_key(cls, input: ApiKeyCreate) -> ApiKeyFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'ApiKeyCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ApiKeyFields(field_name='createApiKey', arguments=cleared_arguments)

    @classmethod
    def set_team_member(cls, input: TeamMemberSet) -> TeamMemberFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'TeamMemberSet!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return TeamMemberFields(field_name='setTeamMember', arguments=cleared_arguments)

    @classmethod
    def remove_team_member(cls, input: TeamMemberRemove) -> UserFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'TeamMemberRemove!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return UserFields(field_name='removeTeamMember', arguments=cleared_arguments)

    @classmethod
    def create_user(cls, input: UserCreate) -> UserFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'UserCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return UserFields(field_name='createUser', arguments=cleared_arguments)

    @classmethod
    def delete_user(cls, user: str) -> UserFields:
        arguments: Dict[str, Dict[str, Any]] = {'user': {'type': 'IdOrKey!', 'value': user}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return UserFields(field_name='deleteUser', arguments=cleared_arguments)

    @classmethod
    def create_role(cls, input: RoleCreate) -> RoleFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'RoleCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return RoleFields(field_name='createRole', arguments=cleared_arguments)

    @classmethod
    def create_team(cls, input: TeamCreate) -> TeamFields:
        arguments: Dict[str, Dict[str, Any]] = {'input': {'type': 'TeamCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return TeamFields(field_name='createTeam', arguments=cleared_arguments)

    @classmethod
    def update_feedback(cls, id: Any, input: FeedbackUpdateInput) -> DirectFeedbackFields:
        arguments: Dict[str, Dict[str, Any]] = {'id': {'type': 'UUID!', 'value': id}, 'input': {'type': 'FeedbackUpdateInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DirectFeedbackFields(field_name='updateFeedback', arguments=cleared_arguments)

    @classmethod
    def add_direct_feedback(cls, completion_id: Any, metric_id: str, input: FeedbackAddInput) -> DirectFeedbackFields:
        arguments: Dict[str, Dict[str, Any]] = {'completionId': {'type': 'UUID!', 'value': completion_id}, 'metricId': {'type': 'IdOrKey!', 'value': metric_id}, 'input': {'type': 'FeedbackAddInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DirectFeedbackFields(field_name='addDirectFeedback', arguments=cleared_arguments)

    @classmethod
    def create_judge(cls, use_case: str, input: JudgeCreate) -> JudgeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'input': {'type': 'JudgeCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return JudgeFields(field_name='createJudge', arguments=cleared_arguments)

    @classmethod
    def create_prebuilt_judge(cls, use_case: str, input: PrebuiltJudgeCreate) -> JudgeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'input': {'type': 'PrebuiltJudgeCreate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return JudgeFields(field_name='createPrebuiltJudge', arguments=cleared_arguments)

    @classmethod
    def update_judge(cls, use_case: str, key: str, input: JudgeUpdate) -> JudgeFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'key': {'type': 'String!', 'value': key}, 'input': {'type': 'JudgeUpdate!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return JudgeFields(field_name='updateJudge', arguments=cleared_arguments)

    @classmethod
    def delete_judge(cls, use_case: str, key: str) -> DeleteConfirmFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'key': {'type': 'String!', 'value': key}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DeleteConfirmFields(field_name='deleteJudge', arguments=cleared_arguments)

    @classmethod
    def create_grader(cls, use_case: str, input: GraderCreateInput) -> GraderFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'input': {'type': 'GraderCreateInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraderFields(field_name='createGrader', arguments=cleared_arguments)

    @classmethod
    def update_grader(cls, use_case: str, id: str, input: GraderUpdateInput) -> GraderFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'id': {'type': 'IdOrKey!', 'value': id}, 'input': {'type': 'GraderUpdateInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraderFields(field_name='updateGrader', arguments=cleared_arguments)

    @classmethod
    def delete_grader(cls, use_case: str, id: str) -> DeleteConfirmFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'id': {'type': 'IdOrKey!', 'value': id}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return DeleteConfirmFields(field_name='deleteGrader', arguments=cleared_arguments)

    @classmethod
    def lock_grader(cls, use_case: str, id: str, locked: bool) -> GraderFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'id': {'type': 'IdOrKey!', 'value': id}, 'locked': {'type': 'Boolean!', 'value': locked}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraderFields(field_name='lockGrader', arguments=cleared_arguments)

    @classmethod
    def create_tool_provider(cls, use_case: str, input: CreateToolProviderInput) -> ToolProviderFields:
        arguments: Dict[str, Dict[str, Any]] = {'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'input': {'type': 'CreateToolProviderInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ToolProviderFields(field_name='createToolProvider', arguments=cleared_arguments)

    @classmethod
    def update_tool_provider(cls, id_or_key: str, use_case: str, input: UpdateToolProviderInput) -> ToolProviderFields:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}, 'input': {'type': 'UpdateToolProviderInput!', 'value': input}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return ToolProviderFields(field_name='updateToolProvider', arguments=cleared_arguments)

    @classmethod
    def delete_tool_provider(cls, id_or_key: str, use_case: str) -> GraphQLField:
        arguments: Dict[str, Dict[str, Any]] = {'idOrKey': {'type': 'IdOrKey!', 'value': id_or_key}, 'useCase': {'type': 'IdOrKey!', 'value': use_case}}
        cleared_arguments = {key: value for key, value in arguments.items() if value['value'] is not None}
        return GraphQLField(field_name='deleteToolProvider', arguments=cleared_arguments)


from uuid import UUID
from pydantic import BaseModel as PydanticBaseModel, ConfigDict

class BaseModel(PydanticBaseModel):
    """@public"""
    model_config = ConfigDict(populate_by_name=True, validate_assignment=True, arbitrary_types_allowed=True, protected_namespaces=())

    def model_dump(self, *args, **kwargs) -> dict:
        data = super().model_dump(*args, **kwargs)
        for key, value in data.items():
            if isinstance(value, UUID):
                data[key] = str(value)
        return data


import enum
import json
from typing import IO, Any, AsyncIterator, Dict, List, Optional, Tuple, TypeVar, Union, cast
from uuid import uuid4
import httpx
from pydantic import BaseModel
from pydantic_core import to_jsonable_python
from .base_model import UNSET, Upload
from .exceptions import GraphQLClientGraphQLMultiError, GraphQLClientHttpError, GraphQLClientInvalidMessageFormat, GraphQLClientInvalidResponseError
try:
    from websockets import ClientConnection
    from websockets import connect as ws_connect
    from websockets.typing import Data, Origin, Subprotocol
except ImportError:
    from contextlib import asynccontextmanager

    @asynccontextmanager
    async def ws_connect(*args, **kwargs):
        raise NotImplementedError("Subscriptions require 'websockets' package.")
        yield
    ClientConnection = Any
    Data = Any
    Origin = Any

    def Subprotocol(*args, **kwargs):
        raise NotImplementedError("Subscriptions require 'websockets' package.")
try:
    from opentelemetry.context import Context
    from opentelemetry.trace import Span, Tracer, get_tracer, set_span_in_context
except ImportError:
    Context = Any
    Span = Any
    Tracer = Any

    def get_tracer(*args, **kwargs) -> Tracer:
        raise NotImplementedError("Telemetry requires 'opentelemetry-api' package.")

    def set_span_in_context(*args, **kwargs):
        raise NotImplementedError("Telemetry requires 'opentelemetry-api' package.")
Self = TypeVar('Self', bound='AsyncBaseClientOpenTelemetry')
GRAPHQL_TRANSPORT_WS = 'graphql-transport-ws'

class GraphQLTransportWSMessageType(str, enum.Enum):
    """@private"""
    CONNECTION_INIT = 'connection_init'
    CONNECTION_ACK = 'connection_ack'
    PING = 'ping'
    PONG = 'pong'
    SUBSCRIBE = 'subscribe'
    NEXT = 'next'
    ERROR = 'error'
    COMPLETE = 'complete'

class AsyncBaseClientOpenTelemetry:
    """@private"""

    def __init__(self, url: str='', headers: Optional[Dict[str, str]]=None, http_client: Optional[httpx.AsyncClient]=None, ws_url: str='', ws_headers: Optional[Dict[str, Any]]=None, ws_origin: Optional[str]=None, ws_connection_init_payload: Optional[Dict[str, Any]]=None, tracer: Optional[Union[str, Tracer]]=None, root_context: Optional[Context]=None, root_span_name: str='GraphQL Operation', ws_root_context: Optional[Context]=None, ws_root_span_name: str='GraphQL Subscription') -> None:
        self.url = url
        self.headers = headers
        self.http_client = http_client if http_client else httpx.AsyncClient(headers=headers)
        self.ws_url = ws_url
        self.ws_headers = ws_headers or {}
        self.ws_origin = Origin(ws_origin) if ws_origin else None
        self.ws_connection_init_payload = ws_connection_init_payload
        self.tracer: Optional[Tracer] = get_tracer(tracer) if isinstance(tracer, str) else tracer
        self.root_context = root_context
        self.root_span_name = root_span_name
        self.ws_root_context = ws_root_context
        self.ws_root_span_name = ws_root_span_name

    async def __aenter__(self: Self) -> Self:
        return self

    async def __aexit__(self, exc_type: object, exc_val: object, exc_tb: object) -> None:
        await self.http_client.aclose()

    async def execute(self, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None, **kwargs: Any) -> httpx.Response:
        if self.tracer:
            return await self._execute_with_telemetry(query=query, operation_name=operation_name, variables=variables, **kwargs)
        return await self._execute(query=query, operation_name=operation_name, variables=variables, **kwargs)

    def get_data(self, response: httpx.Response) -> Dict[str, Any]:
        if not response.is_success:
            raise GraphQLClientHttpError(status_code=response.status_code, response=response)
        try:
            response_json = response.json()
        except ValueError as exc:
            raise GraphQLClientInvalidResponseError(response=response) from exc
        if not isinstance(response_json, dict) or ('data' not in response_json and 'errors' not in response_json):
            raise GraphQLClientInvalidResponseError(response=response)
        data = response_json.get('data')
        errors = response_json.get('errors')
        if errors:
            raise GraphQLClientGraphQLMultiError.from_errors_dicts(errors_dicts=errors, data=data)
        return cast(Dict[str, Any], data)

    async def execute_ws(self, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None, **kwargs: Any) -> AsyncIterator[Dict[str, Any]]:
        if self.tracer:
            generator = self._execute_ws_with_telemetry(query=query, operation_name=operation_name, variables=variables, **kwargs)
        else:
            generator = self._execute_ws(query=query, operation_name=operation_name, variables=variables, **kwargs)
        async for message in generator:
            yield message

    async def _execute(self, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None, **kwargs: Any) -> httpx.Response:
        processed_variables, files, files_map = self._process_variables(variables)
        if files and files_map:
            return await self._execute_multipart(query=query, operation_name=operation_name, variables=processed_variables, files=files, files_map=files_map, **kwargs)
        return await self._execute_json(query=query, operation_name=operation_name, variables=processed_variables, **kwargs)

    def _process_variables(self, variables: Optional[Dict[str, Any]]) -> Tuple[Dict[str, Any], Dict[str, Tuple[str, IO[bytes], str]], Dict[str, List[str]]]:
        if not variables:
            return ({}, {}, {})
        serializable_variables = self._convert_dict_to_json_serializable(variables)
        return self._get_files_from_variables(serializable_variables)

    def _convert_dict_to_json_serializable(self, dict_: Dict[str, Any]) -> Dict[str, Any]:
        return {key: self._convert_value(value) for key, value in dict_.items() if value is not UNSET}

    def _convert_value(self, value: Any) -> Any:
        if isinstance(value, BaseModel):
            return value.model_dump(by_alias=True, exclude_unset=True)
        if isinstance(value, list):
            return [self._convert_value(item) for item in value]
        return value

    def _get_files_from_variables(self, variables: Dict[str, Any]) -> Tuple[Dict[str, Any], Dict[str, Tuple[str, IO[bytes], str]], Dict[str, List[str]]]:
        files_map: Dict[str, List[str]] = {}
        files_list: List[Upload] = []

        def separate_files(path: str, obj: Any) -> Any:
            if isinstance(obj, list):
                nulled_list = []
                for index, value in enumerate(obj):
                    value = separate_files(f'{path}.{index}', value)
                    nulled_list.append(value)
                return nulled_list
            if isinstance(obj, dict):
                nulled_dict = {}
                for key, value in obj.items():
                    value = separate_files(f'{path}.{key}', value)
                    nulled_dict[key] = value
                return nulled_dict
            if isinstance(obj, Upload):
                if obj in files_list:
                    file_index = files_list.index(obj)
                    files_map[str(file_index)].append(path)
                else:
                    file_index = len(files_list)
                    files_list.append(obj)
                    files_map[str(file_index)] = [path]
                return None
            return obj
        nulled_variables = separate_files('variables', variables)
        files: Dict[str, Tuple[str, IO[bytes], str]] = {str(i): (file_.filename, cast(IO[bytes], file_.content), file_.content_type) for i, file_ in enumerate(files_list)}
        return (nulled_variables, files, files_map)

    async def _execute_multipart(self, query: str, operation_name: Optional[str], variables: Dict[str, Any], files: Dict[str, Tuple[str, IO[bytes], str]], files_map: Dict[str, List[str]], **kwargs: Any) -> httpx.Response:
        data = {'operations': json.dumps({'query': query, 'operationName': operation_name, 'variables': variables}, default=to_jsonable_python), 'map': json.dumps(files_map, default=to_jsonable_python)}
        return await self.http_client.post(url=self.url, data=data, files=files, **kwargs)

    async def _execute_json(self, query: str, operation_name: Optional[str], variables: Dict[str, Any], **kwargs: Any) -> httpx.Response:
        headers: Dict[str, str] = {'Content-Type': 'application/json'}
        headers.update(kwargs.get('headers', {}))
        merged_kwargs: Dict[str, Any] = kwargs.copy()
        merged_kwargs['headers'] = headers
        return await self.http_client.post(url=self.url, content=json.dumps({'query': query, 'operationName': operation_name, 'variables': variables}, default=to_jsonable_python), **merged_kwargs)

    async def _execute_ws(self, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None, **kwargs: Any) -> AsyncIterator[Dict[str, Any]]:
        headers = self.ws_headers.copy()
        headers.update(kwargs.get('extra_headers', {}))
        merged_kwargs: Dict[str, Any] = {'origin': self.ws_origin}
        merged_kwargs.update(kwargs)
        merged_kwargs['extra_headers'] = headers
        operation_id = str(uuid4())
        async with ws_connect(self.ws_url, subprotocols=[Subprotocol(GRAPHQL_TRANSPORT_WS)], **merged_kwargs) as websocket:
            await self._send_connection_init(websocket)
            await self._handle_ws_message(await websocket.recv(), websocket, expected_type=GraphQLTransportWSMessageType.CONNECTION_ACK)
            await self._send_subscribe(websocket, operation_id=operation_id, query=query, operation_name=operation_name, variables=variables)
            async for message in websocket:
                data = await self._handle_ws_message(message, websocket)
                if data:
                    yield data

    async def _send_connection_init(self, websocket: ClientConnection) -> None:
        payload: Dict[str, Any] = {'type': GraphQLTransportWSMessageType.CONNECTION_INIT.value}
        if self.ws_connection_init_payload:
            payload['payload'] = self.ws_connection_init_payload
        await websocket.send(json.dumps(payload))

    async def _send_subscribe(self, websocket: ClientConnection, operation_id: str, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None) -> None:
        payload: Dict[str, Any] = {'id': operation_id, 'type': GraphQLTransportWSMessageType.SUBSCRIBE.value, 'payload': {'query': query, 'operationName': operation_name}}
        if variables:
            payload['payload']['variables'] = self._convert_dict_to_json_serializable(variables)
        await websocket.send(json.dumps(payload))

    async def _handle_ws_message(self, message: Data, websocket: ClientConnection, expected_type: Optional[GraphQLTransportWSMessageType]=None) -> Optional[Dict[str, Any]]:
        try:
            message_dict = json.loads(message)
        except json.JSONDecodeError as exc:
            raise GraphQLClientInvalidMessageFormat(message=message) from exc
        type_ = message_dict.get('type')
        payload = message_dict.get('payload', {})
        if not type_ or type_ not in {t.value for t in GraphQLTransportWSMessageType}:
            raise GraphQLClientInvalidMessageFormat(message=message)
        if expected_type and expected_type != type_:
            raise GraphQLClientInvalidMessageFormat(f'Invalid message received. Expected: {expected_type.value}')
        if type_ == GraphQLTransportWSMessageType.NEXT:
            if 'data' not in payload:
                raise GraphQLClientInvalidMessageFormat(message=message)
            return cast(Dict[str, Any], payload['data'])
        if type_ == GraphQLTransportWSMessageType.COMPLETE:
            await websocket.close()
        elif type_ == GraphQLTransportWSMessageType.PING:
            await websocket.send(json.dumps({'type': GraphQLTransportWSMessageType.PONG.value}))
        elif type_ == GraphQLTransportWSMessageType.ERROR:
            raise GraphQLClientGraphQLMultiError.from_errors_dicts(errors_dicts=payload, data=message_dict)
        return None

    async def _execute_with_telemetry(self, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None, **kwargs: Any) -> httpx.Response:
        with self.tracer.start_as_current_span(self.root_span_name, context=self.root_context) as root_span:
            root_span.set_attribute('component', 'GraphQL Client')
            processed_variables, files, files_map = self._process_variables(variables)
            if files and files_map:
                return await self._execute_multipart_with_telemetry(root_span=root_span, query=query, operation_name=operation_name, variables=processed_variables, files=files, files_map=files_map, **kwargs)
            return await self._execute_json_with_telemetry(root_span=root_span, query=query, operation_name=operation_name, variables=processed_variables, **kwargs)

    async def _execute_multipart_with_telemetry(self, root_span: Span, query: str, operation_name: Optional[str], variables: Dict[str, Any], files: Dict[str, Tuple[str, IO[bytes], str]], files_map: Dict[str, List[str]], **kwargs: Any) -> httpx.Response:
        with self.tracer.start_as_current_span('multipart request', context=set_span_in_context(root_span)) as span:
            span.set_attribute('component', 'GraphQL Client')
            serialized_variables = json.dumps(variables, default=to_jsonable_python)
            serialized_map = json.dumps(files_map, default=to_jsonable_python)
            span.set_attribute('query', query)
            span.set_attribute('operationName', operation_name or '')
            span.set_attribute('variables', serialized_variables)
            span.set_attribute('map', serialized_map)
            return await self._execute_multipart(query=query, operation_name=operation_name, variables=variables, files=files, files_map=files_map, **kwargs)

    async def _execute_json_with_telemetry(self, root_span: Span, query: str, operation_name: Optional[str], variables: Dict[str, Any], **kwargs: Any) -> httpx.Response:
        with self.tracer.start_as_current_span('json request', context=set_span_in_context(root_span)) as span:
            span.set_attribute('component', 'GraphQL Client')
            serialized_variables = json.dumps(variables, default=to_jsonable_python)
            span.set_attribute('query', query)
            span.set_attribute('operationName', operation_name or '')
            span.set_attribute('variables', serialized_variables)
            return await self._execute_json(query=query, operation_name=operation_name, variables=variables, **kwargs)

    async def _execute_ws_with_telemetry(self, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None, **kwargs: Any) -> AsyncIterator[Dict[str, Any]]:
        with self.tracer.start_as_current_span(self.ws_root_span_name, context=self.ws_root_context) as root_span:
            root_span.set_attribute('component', 'GraphQL Client')
            headers = self.ws_headers.copy()
            headers.update(kwargs.get('extra_headers', {}))
            merged_kwargs: Dict[str, Any] = {'origin': self.ws_origin}
            merged_kwargs.update(kwargs)
            merged_kwargs['extra_headers'] = headers
            operation_id = str(uuid4())
            async with ws_connect(self.ws_url, subprotocols=[Subprotocol(GRAPHQL_TRANSPORT_WS)], **merged_kwargs) as websocket:
                await self._send_connection_init_with_telemetry(root_span=root_span, websocket=websocket)
                await self._handle_ws_message_with_telemetry(root_span=root_span, message=await websocket.recv(), websocket=websocket, expected_type=GraphQLTransportWSMessageType.CONNECTION_ACK)
                await self._send_subscribe_with_telemetry(root_span=root_span, websocket=websocket, operation_id=operation_id, query=query, operation_name=operation_name, variables=variables)
                async for message in websocket:
                    data = await self._handle_ws_message_with_telemetry(root_span=root_span, message=message, websocket=websocket)
                    if data:
                        yield data

    async def _send_connection_init_with_telemetry(self, root_span: Span, websocket: ClientConnection) -> None:
        with self.tracer.start_as_current_span('connection init', context=set_span_in_context(root_span)) as span:
            span.set_attribute('component', 'GraphQL Client')
            span.set_attribute('type', GraphQLTransportWSMessageType.CONNECTION_INIT.value)
            if self.ws_connection_init_payload:
                span.set_attribute('payload', json.dumps(self.ws_connection_init_payload))
            await self._send_connection_init(websocket=websocket)

    async def _send_subscribe_with_telemetry(self, root_span: Span, websocket: ClientConnection, operation_id: str, query: str, operation_name: Optional[str]=None, variables: Optional[Dict[str, Any]]=None) -> None:
        with self.tracer.start_as_current_span('subscribe', context=set_span_in_context(root_span)) as span:
            span.set_attribute('component', 'GraphQL Client')
            span.set_attribute('id', operation_id)
            span.set_attribute('type', GraphQLTransportWSMessageType.SUBSCRIBE.value)
            span.set_attribute('query', query)
            span.set_attribute('operationName', operation_name or '')
            if variables:
                span.set_attribute('variables', json.dumps(self._convert_dict_to_json_serializable(variables)))
            await self._send_subscribe(websocket=websocket, operation_id=operation_id, query=query, operation_name=operation_name, variables=variables)

    async def _handle_ws_message_with_telemetry(self, root_span: Span, message: Data, websocket: ClientConnection, expected_type: Optional[GraphQLTransportWSMessageType]=None) -> Optional[Dict[str, Any]]:
        with self.tracer.start_as_current_span('received message', context=set_span_in_context(root_span)) as span:
            span.set_attribute('component', 'GraphQL Client')
            try:
                message_dict = json.loads(message)
            except json.JSONDecodeError as exc:
                raise GraphQLClientInvalidMessageFormat(message=message) from exc
            type_ = message_dict.get('type')
            payload = message_dict.get('payload', {})
            span.set_attribute('type', type_)
            if not type_ or type_ not in {t.value for t in GraphQLTransportWSMessageType}:
                raise GraphQLClientInvalidMessageFormat(message=message)
            if expected_type and expected_type != type_:
                raise GraphQLClientInvalidMessageFormat(f'Invalid message received. Expected: {expected_type.value}')
            if type_ == GraphQLTransportWSMessageType.NEXT:
                if 'data' not in payload:
                    raise GraphQLClientInvalidMessageFormat(message=message)
                return cast(Dict[str, Any], payload['data'])
            if type_ == GraphQLTransportWSMessageType.COMPLETE:
                await websocket.close()
            elif type_ == GraphQLTransportWSMessageType.PING:
                await websocket.send(json.dumps({'type': GraphQLTransportWSMessageType.PONG.value}))
            elif type_ == GraphQLTransportWSMessageType.ERROR:
                raise GraphQLClientGraphQLMultiError.from_errors_dicts(errors_dicts=payload, data=message_dict)
            return None


from .patch import *
from .client import Adaptive, AsyncAdaptive
from adaptive_sdk import resources, input_types, graphql_client, rest, external


__version__ = "0.0.1b1"
__all__ = [
    "Adaptive",
    "AsyncAdaptive",
    "resources",
    "input_types",
    "graphql_client",
    "rest",
    "external",
]



from typing import Optional
from pydantic import Field
from .base_model import BaseModel
from .fragments import AbCampaignDetailData, AbCampaignReportData

class DescribeAbCampaign(BaseModel):
    """@public"""
    ab_campaign: Optional['DescribeAbCampaignAbCampaign'] = Field(alias='abCampaign')

class DescribeAbCampaignAbCampaign(AbCampaignDetailData):
    """@public"""
    report: 'DescribeAbCampaignAbCampaignReport'

class DescribeAbCampaignAbCampaignReport(AbCampaignReportData):
    """@public"""
    pass
DescribeAbCampaign.model_rebuild()
DescribeAbCampaignAbCampaign.model_rebuild()


from pydantic import Field
from .base_model import BaseModel
from .fragments import UseCaseData

class CreateUseCase(BaseModel):
    """@public"""
    create_use_case: 'CreateUseCaseCreateUseCase' = Field(alias='createUseCase')

class CreateUseCaseCreateUseCase(UseCaseData):
    """@public"""
    pass
CreateUseCase.model_rebuild()


"""
Wrapper for JobData that adds eval_result functionality without modifying autogenerated code.
"""

from __future__ import annotations
from typing import TYPE_CHECKING
from pydantic import BaseModel, Field

from adaptive_sdk.graphql_client import JobData, JobArtifactKind, JobDataDetailsArtifactsByproductsEvaluationByproducts

if TYPE_CHECKING:
    pass


class EvalMetricScore(BaseModel):
    """A single metric score for a model in an evaluation."""

    metric_key: str = Field(description="The key of the metric")
    metric_name: str = Field(description="The human-readable name of the metric")
    mean: float = Field(description="The mean score for this metric")
    min: float = Field(description="The minimum score for this metric")
    max: float = Field(description="The maximum score for this metric")
    stddev: float = Field(description="The standard deviation of scores")
    count: int = Field(description="The number of samples evaluated")
    feedback_count: int = Field(description="The number of feedbacks received")


class ModelEvalResult(BaseModel):
    """Evaluation results for a single model."""

    model_key: str = Field(description="The key of the model that was evaluated")
    model_name: str = Field(description="The human-readable name of the model")
    scores: list[EvalMetricScore] = Field(description="List of metric scores for this model", default_factory=list)


class JobDataPlus(JobData):
    """
    Extended JobData that adds eval_result() method.
    Inherits all attributes and methods from JobData.
    """

    @classmethod
    def from_job_data(cls, job_data: JobData) -> "JobDataPlus":
        """
        Create a JobDataWrapper from a JobData instance.

        Args:
            job_data: The JobData instance to wrap

        Returns:
            A new JobDataWrapper with all the same data
        """
        # Use model_validate to create a new instance with the same data
        return cls.model_validate(job_data.model_dump())

    @property
    def eval_results(self) -> list[ModelEvalResult] | None:
        """
        Extract and organize evaluation results from this job.

        Returns:
            A list of ModelEvalResult objects (one per model), or None if no eval results exist
        """
        if not self.details or not self.details.artifacts:
            return None

        # Dictionary to group scores by model
        model_results: dict[str, ModelEvalResult] = {}

        # Iterate through all artifacts looking for evaluation byproducts
        for artifact in self.details.artifacts:
            if not artifact.byproducts:
                continue

            # Check if this is an EvaluationByproducts with eval_results
            if artifact.kind == JobArtifactKind.EVALUATION:
                if artifact.byproducts is not None:
                    assert isinstance(artifact.byproducts, JobDataDetailsArtifactsByproductsEvaluationByproducts)

                    for eval_result in artifact.byproducts.eval_results:
                        model_key = eval_result.model_service.key
                        model_name = eval_result.model_service.name

                        # Create or get the model result entry
                        if model_key not in model_results:
                            model_results[model_key] = ModelEvalResult(
                                model_key=model_key, model_name=model_name, scores=[]
                            )

                        # Add the metric score to this model's results
                        metric_score = EvalMetricScore(
                            metric_key=eval_result.metric.key,
                            metric_name=eval_result.metric.name,
                            mean=eval_result.mean,
                            min=eval_result.min,
                            max=eval_result.max,
                            stddev=eval_result.stddev,
                            count=eval_result.count,
                            feedback_count=eval_result.feedback_count,
                        )
                        model_results[model_key].scores.append(metric_score)

        if not model_results:
            return None

        return list(model_results.values())



from typing import Optional
from pydantic import Field
from .base_model import BaseModel

class DeleteJudge(BaseModel):
    """@public"""
    delete_judge: 'DeleteJudgeDeleteJudge' = Field(alias='deleteJudge')

class DeleteJudgeDeleteJudge(BaseModel):
    """@public"""
    success: bool
    details: Optional[str]
DeleteJudge.model_rebuild()


from typing import Any, List
from pydantic import Field
from .base_model import BaseModel

class ListRoles(BaseModel):
    """@public"""
    roles: List['ListRolesRoles']

class ListRolesRoles(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')
    permissions: List[str]
ListRoles.model_rebuild()


from typing import Any
from pydantic import Field
from .base_model import BaseModel

class CreateTeam(BaseModel):
    """@public"""
    create_team: 'CreateTeamCreateTeam' = Field(alias='createTeam')

class CreateTeamCreateTeam(BaseModel):
    """@public"""
    id: Any
    key: str
    name: str
    created_at: int = Field(alias='createdAt')
CreateTeam.model_rebuild()


from pydantic import Field
from .base_model import BaseModel
from .fragments import ModelServiceData

class UpdateModel(BaseModel):
    """@public"""
    update_model_service: 'UpdateModelUpdateModelService' = Field(alias='updateModelService')

class UpdateModelUpdateModelService(ModelServiceData):
    """@public"""
    pass
UpdateModel.model_rebuild()


from typing import Any, Optional
from pydantic import Field
from .base_model import BaseModel
from .enums import SessionStatus

class DatasetUploadProcessingStatus(BaseModel):
    """@public"""
    dataset_upload_processing_status: 'DatasetUploadProcessingStatusDatasetUploadProcessingStatus' = Field(alias='datasetUploadProcessingStatus')

class DatasetUploadProcessingStatusDatasetUploadProcessingStatus(BaseModel):
    """@public"""
    dataset_id: Any = Field(alias='datasetId')
    status: SessionStatus
    total_parts: int = Field(alias='totalParts')
    processed_parts: int = Field(alias='processedParts')
    progress: float
    error: Optional[str]
DatasetUploadProcessingStatus.model_rebuild()


from pydantic import Field
from .base_model import BaseModel
from .fragments import DatasetData

class LoadDataset(BaseModel):
    """@public"""
    create_dataset: 'LoadDatasetCreateDataset' = Field(alias='createDataset')

class LoadDatasetCreateDataset(DatasetData):
    """@public"""
    pass
LoadDataset.model_rebuild()


from .typed_dicts import (
    ChatMessage,
    ComparisonCompletion,
    CompletionComparisonFilterInput,
    CompletionFeedbackFilterInput,
    CompletionLabelFilter,
    CompletionSource,
    CursorPageInput,
    InteractionFeedbackDict,
    JudgeExampleInput,
    ListCompletionsFilterInput,
    ModelComputeConfigInput,
    ModelFilter,
    ModelPlacementInput,
    TimeRange,
    Order,
)

__all__ = [
    "ChatMessage",
    "ComparisonCompletion",
    "CompletionComparisonFilterInput",
    "CompletionFeedbackFilterInput",
    "CompletionLabelFilter",
    "CompletionSource",
    "CursorPageInput",
    "ListCompletionsFilterInput",
    "InteractionFeedbackDict",
    "JudgeExampleInput",
    "ModelComputeConfigInput",
    "ModelFilter",
    "ModelPlacementInput",
    "TimeRange",
    "Order",
]



from typing import Any, Dict, List, Optional, Union
import httpx

class GraphQLClientError(Exception):
    """@private
Base exception."""

class GraphQLClientHttpError(GraphQLClientError):
    """@private"""

    def __init__(self, status_code: int, response: httpx.Response) -> None:
        self.status_code = status_code
        self.response = response

    def __str__(self) -> str:
        return f'HTTP status code: {self.status_code}'

class GraphQLClientInvalidResponseError(GraphQLClientError):
    """@private"""

    def __init__(self, response: httpx.Response) -> None:
        self.response = response

    def __str__(self) -> str:
        return 'Invalid response format.'

class GraphQLClientGraphQLError(GraphQLClientError):
    """@private"""

    def __init__(self, message: str, locations: Optional[List[Dict[str, int]]]=None, path: Optional[List[str]]=None, extensions: Optional[Dict[str, object]]=None, original: Optional[Dict[str, object]]=None):
        self.message = message
        self.locations = locations
        self.path = path
        self.extensions = extensions
        self.original = original

    def __str__(self) -> str:
        return self.message

    @classmethod
    def from_dict(cls, error: Dict[str, Any]) -> 'GraphQLClientGraphQLError':
        return cls(message=error['message'], locations=error.get('locations'), path=error.get('path'), extensions=error.get('extensions'), original=error)

class GraphQLClientGraphQLMultiError(GraphQLClientError):
    """@private"""

    def __init__(self, errors: List[GraphQLClientGraphQLError], data: Optional[Dict[str, Any]]=None):
        self.errors = errors
        self.data = data

    def __str__(self) -> str:
        return '; '.join((str(e) for e in self.errors))

    @classmethod
    def from_errors_dicts(cls, errors_dicts: List[Dict[str, Any]], data: Optional[Dict[str, Any]]=None) -> 'GraphQLClientGraphQLMultiError':
        return cls(errors=[GraphQLClientGraphQLError.from_dict(e) for e in errors_dicts], data=data)

class GraphQLClientInvalidMessageFormat(GraphQLClientError):
    """@private"""

    def __init__(self, message: Union[str, bytes]) -> None:
        self.message = message

    def __str__(self) -> str:
        return 'Invalid message format.'


from __future__ import annotations
from typing import TYPE_CHECKING, List

from adaptive_sdk.graphql_client import CreateTeamCreateTeam, ListTeamsTeams, TeamCreate

from .base_resource import SyncAPIResource, AsyncAPIResource, UseCaseResource

if TYPE_CHECKING:
    from adaptive_sdk.client import Adaptive, AsyncAdaptive


class Teams(SyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to manage teams.
    """

    def __init__(self, client: Adaptive) -> None:
        SyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    def list(self) -> List[ListTeamsTeams]:
        return self._gql_client.list_teams().teams

    def create(self, key: str, name: str | None = None) -> CreateTeamCreateTeam:
        input = TeamCreate(key=key, name=name or key)
        return self._gql_client.create_team(input).create_team


class AsyncTeams(AsyncAPIResource, UseCaseResource):  # type: ignore[misc]
    """
    Resource to manage teams.
    """

    def __init__(self, client: AsyncAdaptive) -> None:
        AsyncAPIResource.__init__(self, client)
        UseCaseResource.__init__(self, client)

    async def list(self) -> List[ListTeamsTeams]:
        return (await self._gql_client.list_teams()).teams

    async def create(self, key: str, name: str | None = None) -> CreateTeamCreateTeam:
        input = TeamCreate(key=key, name=name or key)
        return (await self._gql_client.create_team(input)).create_team



from typing import List
from pydantic import Field
from .base_model import BaseModel
from .fragments import UseCaseData

class ListUseCases(BaseModel):
    """@public"""
    use_cases: List['ListUseCasesUseCases'] = Field(alias='useCases')

class ListUseCasesUseCases(UseCaseData):
    """@public"""
    pass
ListUseCases.model_rebuild()


from typing import List
from .base_model import BaseModel
from .fragments import GraderData

class ListGraders(BaseModel):
    """@public"""
    graders: List['ListGradersGraders']

class ListGradersGraders(GraderData):
    """@public"""
    pass
ListGraders.model_rebuild()


from pydantic import BaseModel as PydanticBaseModel, ConfigDict


class BaseModel(PydanticBaseModel):
    """@private"""

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        arbitrary_types_allowed=True,
        protected_namespaces=(),
    )



from pydantic import Field
from .base_model import BaseModel

class ResizeInferencePartition(BaseModel):
    """@public"""
    resize_inference_partition: str = Field(alias='resizeInferencePartition')


from typing import Any, Dict, List, Optional, Set, Tuple, Union
from graphql import ArgumentNode, FieldNode, InlineFragmentNode, NamedTypeNode, NameNode, SelectionSetNode, VariableNode

class GraphQLArgument:
    """@private
Represents a GraphQL argument and allows conversion to an AST structure."""

    def __init__(self, argument_name: str, argument_value: Any) -> None:
        self._name = argument_name
        self._value = argument_value

    def to_ast(self) -> ArgumentNode:
        """Converts the argument to an ArgumentNode AST object."""
        return ArgumentNode(name=NameNode(value=self._name), value=VariableNode(name=NameNode(value=self._value)))

class GraphQLField:
    """@private
Represents a GraphQL field with its name, arguments, subfields, alias,
and inline fragments.

Attributes:
    formatted_variables (Dict[str, Dict[str, Any]]): The formatted arguments
    of the GraphQL field."""

    def __init__(self, field_name: str, arguments: Optional[Dict[str, Dict[str, Any]]]=None) -> None:
        self._field_name = field_name
        self._variables = arguments or {}
        self.formatted_variables: Dict[str, Dict[str, Any]] = {}
        self._subfields: List[GraphQLField] = []
        self._alias: Optional[str] = None
        self._inline_fragments: Dict[str, Tuple[GraphQLField, ...]] = {}

    def alias(self, alias: str) -> 'GraphQLField':
        """Sets an alias for the GraphQL field and returns the instance."""
        self._alias = alias
        return self

    def _build_field_name(self) -> str:
        """Builds the field name, including the alias if present."""
        return f'{self._alias}: {self._field_name}' if self._alias else self._field_name

    def _build_selections(self, idx: int, used_names: Set[str]) -> List[Union[FieldNode, InlineFragmentNode]]:
        """Builds the selection set for the current GraphQL field,
        including subfields and inline fragments."""
        selections: List[Union[FieldNode, InlineFragmentNode]] = [subfield.to_ast(idx, used_names) for subfield in self._subfields]
        for name, subfields in self._inline_fragments.items():
            selections.append(InlineFragmentNode(type_condition=NamedTypeNode(name=NameNode(value=name)), selection_set=SelectionSetNode(selections=[subfield.to_ast(idx, used_names) for subfield in subfields])))
        return selections

    def _format_variable_name(self, idx: int, var_name: str, used_names: Set[str]) -> str:
        """Generates a unique variable name by appending an index and,
        if necessary, an additional counter to avoid duplicates."""
        base_name = f'{var_name}_{idx}'
        unique_name = base_name
        counter = 1
        while unique_name in used_names:
            unique_name = f'{base_name}_{counter}'
            counter += 1
        used_names.add(unique_name)
        return unique_name

    def _collect_all_variables(self, idx: int, used_names: Set[str]) -> None:
        """
        Collects and formats all variables for the current GraphQL field,
        ensuring unique names.
        """
        self.formatted_variables = {}
        for k, v in self._variables.items():
            unique_name = self._format_variable_name(idx, k, used_names)
            self.formatted_variables[unique_name] = {'name': k, 'type': v['type'], 'value': v['value']}

    def to_ast(self, idx: int, used_names: Optional[Set[str]]=None) -> FieldNode:
        """Converts the current GraphQL field to an AST (Abstract Syntax Tree) node."""
        if used_names is None:
            used_names = set()
        self._collect_all_variables(idx, used_names)
        return FieldNode(name=NameNode(value=self._build_field_name()), arguments=[GraphQLArgument(v['name'], k).to_ast() for k, v in self.formatted_variables.items()], selection_set=SelectionSetNode(selections=self._build_selections(idx, used_names)) if self._subfields or self._inline_fragments else None)

    def get_formatted_variables(self) -> Dict[str, Dict[str, Any]]:
        """
        Retrieves all formatted variables for the current GraphQL field,
        including those from subfields and inline fragments.
        """
        formatted_variables = self.formatted_variables.copy()
        for subfield in self._subfields:
            subfield.get_formatted_variables()
            formatted_variables.update(subfield.formatted_variables)
        for subfields in self._inline_fragments.values():
            for subfield in subfields:
                subfield.get_formatted_variables()
                formatted_variables.update(subfield.formatted_variables)
        return formatted_variables


from __future__ import annotations
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from adaptive_sdk.base_client import BaseAsyncClient, BaseSyncClient, UseCaseClient


class SyncAPIResource:
    def __init__(self, client: BaseSyncClient) -> None:
        self._client = client
        self._rest_client = client._rest_client
        self._gql_client = client._gql_client


class AsyncAPIResource:
    def __init__(self, client: BaseAsyncClient) -> None:
        self._client = client
        self._rest_client = client._rest_client
        self._gql_client = client._gql_client


class UseCaseResource:
    def __init__(self, client: UseCaseClient) -> None:
        self._client = client

    def use_case_key(self, use_case: str | None) -> str:
        target_use_case = use_case or self._client.default_use_case
        if target_use_case is None:
            raise ValueError(
                """This method is use case-specific. Either set a default use case for the client with `client.set_default_use_case`, \
or explicitly pass `use_case` as an input parameter."""
            )
        elif not bool(target_use_case.strip()):
            raise ValueError("use_case must be a non-empty string")

        return target_use_case

    def optional_use_case_key(self, use_case: str | None) -> str | None:
        if use_case:
            return use_case
        elif self._client.default_use_case:
            return self._client.default_use_case
        else:
            return None



from .abtests import ABTests, AsyncABTests
from .chat import Chat, AsyncChat
from .compute_pools import ComputePools, AsyncComputePools  # type: ignore[attr-defined]
from .recipes import Recipes, AsyncRecipes
from .datasets import Datasets, AsyncDatasets
from .embeddings import Embeddings, AsyncEmbeddings
from .feedback import Feedback, AsyncFeedback
from .interactions import Interactions, AsyncInteractions
from .jobs import Jobs, AsyncJobs
from .graders import Graders, AsyncGraders
from .models import Models, AsyncModels
from .permissions import Permissions, AsyncPermissions
from .roles import Roles, AsyncRoles
from .teams import Teams, AsyncTeams
from .use_cases import UseCase, AsyncUseCase
from .users import Users, AsyncUsers

__all__ = [
    "ABTests",
    "Chat",
    "ComputePools",
    "Recipes",
    "Datasets",
    "Embeddings",
    "Feedback",
    "Interactions",
    "Jobs",
    "Graders",
    "Models",
    "Permissions",
    "Roles",
    "Teams",
    "UseCase",
    "Users",
    "AsyncABTests",
    "AsyncChat",
    "AsyncComputePools",
    "AsyncRecipes",
    "AsyncDatasets",
    "AsyncEmbeddings",
    "AsyncFeedback",
    "AsyncInteractions",
    "AsyncJobs",
    "AsyncGraders",
    "AsyncModels",
    "AsyncPermissions",
    "AsyncRoles",
    "AsyncTeams",
    "AsyncUseCase",
    "AsyncUsers",
]
